==> zarf-v0.28.3.tar.gz: zarf-0.28.3/src/cmd/destroy.go <==
// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: 2021-Present The Zarf Authors

// Package cmd contains the CLI commands for Zarf.
package cmd

import (
	"errors"
	"os"
	"regexp"

	"github.com/defenseunicorns/zarf/src/config"
	"github.com/defenseunicorns/zarf/src/config/lang"
	"github.com/defenseunicorns/zarf/src/internal/cluster"
	"github.com/defenseunicorns/zarf/src/internal/packager/helm"
	"github.com/defenseunicorns/zarf/src/pkg/message"
	"github.com/defenseunicorns/zarf/src/pkg/utils"
	"github.com/defenseunicorns/zarf/src/pkg/utils/exec"

	"github.com/spf13/cobra"
)

var confirmDestroy bool
var removeComponents bool

var destroyCmd = &cobra.Command{
	Use:     "destroy --confirm",
	Aliases: []string{"d"},
	Short:   lang.CmdDestroyShort,
	Long:    lang.CmdDestroyLong,
	Run: func(cmd *cobra.Command, args []string) {
		c, err := cluster.NewClusterWithWait(cluster.DefaultTimeout, true)
		if err != nil {
			message.Fatalf(err, lang.ErrNoClusterConnection)
		}

		// NOTE: If 'zarf init' failed to deploy the k3s component (or if we're looking at the wrong kubeconfig)
		//       there will be no zarf-state to load and the struct will be empty. In these cases, if we can find
		//       the scripts to remove k3s, we will still try to remove a locally installed k3s cluster
		state, err := c.LoadZarfState()
		if err != nil {
			message.WarnErr(err, lang.ErrLoadState)
		}

		// If Zarf deployed the cluster, burn it all down
		if state.ZarfAppliance || (state.Distro == "") {
			// Check if we have the scripts to destroy everything
			fileInfo, err := os.Stat(config.ZarfCleanupScriptsPath)
			if errors.Is(err, os.ErrNotExist) || !fileInfo.IsDir() {
				message.Fatalf(lang.CmdDestroyErrNoScriptPath, config.ZarfCleanupScriptsPath)
			}

			// Run all the scripts!
			pattern := regexp.MustCompile(`(?mi)zarf-clean-.+\.sh$`)
			scripts, _ := utils.RecursiveFileList(config.ZarfCleanupScriptsPath, pattern, true)
			// Iterate over all matching zarf-clean scripts and exec them
			for _, script := range scripts {
				// Run the matched script
				err := exec.CmdWithPrint(script)
				if errors.Is(err, os.ErrPermission) {
					message.Warnf(lang.CmdDestroyErrScriptPermissionDenied, script)

					// Don't remove scripts we can't execute so the user can try to manually run
					continue
				} else if err != nil {
					message.Debugf("Received error when trying to execute the script (%s): %#v", script, err)
				}

				// Try to remove the script, but ignore any errors
				_ = os.Remove(script)
			}
		} else {
			// Perform chart uninstallation
			helmCfg := helm.Helm{}
			helmCfg.Destroy(removeComponents)

			// If Zarf didn't deploy the cluster, only delete the ZarfNamespace
			c.DeleteZarfNamespace()

			// Remove zarf agent labels and secrets from namespaces Zarf doesn't manage
			c.StripZarfLabelsAndSecretsFromNamespaces()
		}
	},
}

func init() {
	rootCmd.AddCommand(destroyCmd)

	// Still going to require a flag for destroy confirm, no viper oopsies here
	destroyCmd.Flags().BoolVar(&confirmDestroy, "confirm", false, lang.CmdDestroyFlagConfirm)
	destroyCmd.Flags().BoolVar(&removeComponents, "remove-components", false, lang.CmdDestroyFlagRemoveComponents)
	_ = destroyCmd.MarkFlagRequired("confirm")
}

==> zarf-v0.31.0.tar.gz: zarf-0.31.0/src/cmd/destroy.go <==
// SPDX-License-Identifier: Apache-2.0
// SPDX-FileCopyrightText: 2021-Present The Zarf Authors

// Package cmd contains the CLI commands for Zarf.
package cmd

import (
	"errors"
	"os"
	"regexp"

	"github.com/defenseunicorns/zarf/src/config"
	"github.com/defenseunicorns/zarf/src/config/lang"
	"github.com/defenseunicorns/zarf/src/internal/cluster"
	"github.com/defenseunicorns/zarf/src/internal/packager/helm"
	"github.com/defenseunicorns/zarf/src/pkg/message"
	"github.com/defenseunicorns/zarf/src/pkg/utils"
	"github.com/defenseunicorns/zarf/src/pkg/utils/exec"

	"github.com/spf13/cobra"
)

var confirmDestroy bool
var removeComponents bool

var destroyCmd = &cobra.Command{
	Use:     "destroy --confirm",
	Aliases: []string{"d"},
	Short:   lang.CmdDestroyShort,
	Long:    lang.CmdDestroyLong,
	Run: func(cmd *cobra.Command, args []string) {
		c, err := cluster.NewClusterWithWait(cluster.DefaultTimeout)
		if err != nil {
			message.Fatalf(err, lang.ErrNoClusterConnection)
		}

		// NOTE: If 'zarf init' failed to deploy the k3s component (or if we're looking at the wrong kubeconfig)
		//       there will be no zarf-state to load and the struct will be empty. In these cases, if we can find
		//       the scripts to remove k3s, we will still try to remove a locally installed k3s cluster
		state, err := c.LoadZarfState()
		if err != nil {
			message.WarnErr(err, lang.ErrLoadState)
		}

		// If Zarf deployed the cluster, burn it all down
		if state.ZarfAppliance || (state.Distro == "") {
			// Check if we have the scripts to destroy everything
			fileInfo, err := os.Stat(config.ZarfCleanupScriptsPath)
			if errors.Is(err, os.ErrNotExist) || !fileInfo.IsDir() {
				message.Fatalf(lang.CmdDestroyErrNoScriptPath, config.ZarfCleanupScriptsPath)
			}

			// Run all the scripts!
			pattern := regexp.MustCompile(`(?mi)zarf-clean-.+\.sh$`)
			scripts, _ := utils.RecursiveFileList(config.ZarfCleanupScriptsPath, pattern, true)
			// Iterate over all matching zarf-clean scripts and exec them
			for _, script := range scripts {
				// Run the matched script
				err := exec.CmdWithPrint(script)
				if errors.Is(err, os.ErrPermission) {
					message.Warnf(lang.CmdDestroyErrScriptPermissionDenied, script)

					// Don't remove scripts we can't execute so the user can try to manually run
					continue
				} else if err != nil {
					message.Debugf("Received error when trying to execute the script (%s): %#v", script, err)
				}

				// Try to remove the script, but ignore any errors
				_ = os.Remove(script)
			}
		} else {
			// Perform chart uninstallation
			helmCfg := helm.Helm{}
			helmCfg.Destroy(removeComponents)

			// If Zarf didn't deploy the cluster, only delete the ZarfNamespace
			c.DeleteZarfNamespace()

			// Remove zarf agent labels and secrets from namespaces Zarf doesn't manage
			c.StripZarfLabelsAndSecretsFromNamespaces()
		}
	},
}

func init() {
	rootCmd.AddCommand(destroyCmd)

	// Still going to require a flag for destroy confirm, no viper oopsies here
	destroyCmd.Flags().BoolVar(&confirmDestroy, "confirm", false, lang.CmdDestroyFlagConfirm)
	destroyCmd.Flags().BoolVar(&removeComponents, "remove-components", false, lang.CmdDestroyFlagRemoveComponents)
	_ = destroyCmd.MarkFlagRequired("confirm")
}
